from nacl.encoding import RawEncoder
from nacl.hash import blake2b
from nacl.public import PrivateKey
from nacl.pwhash import argon2id
from nacl.pwhash.argon2id import (
    MEMLIMIT_INTERACTIVE,
    MEMLIMIT_MODERATE,
    MEMLIMIT_SENSITIVE,
    OPSLIMIT_INTERACTIVE,
    OPSLIMIT_MODERATE,
    OPSLIMIT_SENSITIVE,
    SALTBYTES,
)

__all__ = ["KDF_PROFILES", "KdfProfile", "hash_salt", "kdf"]

type KdfProfile = tuple[int, int]

KDF_PROFILES: dict[str, KdfProfile] = {
    "interactive": (OPSLIMIT_INTERACTIVE, MEMLIMIT_INTERACTIVE),
    "moderate": (OPSLIMIT_MODERATE, MEMLIMIT_MODERATE),
    "sensitive": (OPSLIMIT_SENSITIVE, MEMLIMIT_SENSITIVE),
}

SALT_MOD = b"""And here I solemnly protest I have no intention to vilify or asperse any
one; for though everything is copied from the book of nature, and scarce a character or
action produced which I have not taken from my own observations or experience; yet I
have used the utmost care to obscure the persons by such different circumstances,
degrees, and colors, that it will be impossible to guess at them with any degree of
certainty;
(c) Henry Fielding
"""


def hash_salt(salt: bytes) -> bytes:
    return blake2b(data=salt, digest_size=SALTBYTES, encoder=RawEncoder)


def kdf(password: bytes, profile: KdfProfile) -> bytes:
    salt = hash_salt(password + SALT_MOD)
    ops, mem = profile
    return argon2id.kdf(
        size=PrivateKey.SIZE,
        password=password,
        salt=salt,
        opslimit=ops,
        memlimit=mem,
        encoder=RawEncoder,
    )



from .cli_options import CliOptions
from .cli_reader import CliReader
from .cli_types import CliTypes
from .cli_writer import CliWriter
from .main import cli

__all__ = ["CliOptions", "CliReader", "CliTypes", "CliWriter", "cli"]



from collections.abc import Callable
from compression import bz2, lzma, zlib, zstd

import brotli

__all__ = ["ARCHIVERS", "UNARCHIVERS", "Archiver"]


type Archiver = Callable[[bytes], bytes]


def compress_brotli(data: bytes) -> bytes:
    return brotli.compress(data, quality=11)


def compress_zstd(data: bytes) -> bytes:
    param = zstd.CompressionParameter
    _, upper = param.compression_level.bounds()
    options = {
        param.compression_level: upper,
        param.checksum_flag: 0,
        param.content_size_flag: 0,
        param.dict_id_flag: 0,
    }
    return zstd.compress(data, options=options)  # type: ignore[arg-type]


def compress_zlib(data: bytes) -> bytes:
    return zlib.compress(data, level=9)


def compress_bz2(data: bytes) -> bytes:
    return bz2.compress(data, compresslevel=9)


def compress_lzma(data: bytes) -> bytes:
    return lzma.compress(
        data,
        format=lzma.FORMAT_ALONE,
        check=lzma.CHECK_NONE,
        preset=lzma.PRESET_EXTREME,
    )


def decompress_brotli(data: bytes) -> bytes:
    return brotli.decompress(data)


def decompress_zstd(data: bytes) -> bytes:
    return zstd.decompress(data)


def decompress_zlib(data: bytes) -> bytes:
    return zlib.decompress(data)


def decompress_bz2(data: bytes) -> bytes:
    return bz2.decompress(data)


def decompress_lzma(data: bytes) -> bytes:
    return lzma.decompress(data, format=lzma.FORMAT_ALONE)


ARCHIVERS: dict[str, Archiver | None] = {
    "brotli": compress_brotli,
    "zstd": compress_zstd,
    "zlib": compress_zlib,
    "bz2": compress_bz2,
    "lzma": compress_lzma,
    "raw": None,
}

UNARCHIVERS: dict[str, Archiver | None] = {
    "brotli": decompress_brotli,
    "zstd": decompress_zstd,
    "zlib": decompress_zlib,
    "bz2": decompress_bz2,
    "lzma": decompress_lzma,
    "raw": None,
}



import hashlib
import time
from typing import TYPE_CHECKING

from pyotp import TOTP

if TYPE_CHECKING:
    from rtty_soda.interfaces import Reader

__all__ = ["GoogleAuthenticator"]


class GoogleAuthenticator:
    def __init__(self, key: Reader) -> None:
        self.secret = key.read_str().strip()
        self.totp = TOTP(s=self.secret, digits=6, digest=hashlib.sha1, interval=30)

    def get_code(self) -> str:
        code = self.totp.now()
        return f"{code[:3]} {code[3:]}"

    def get_remaining_seconds(self) -> int:
        interval = self.totp.interval
        return interval - int(time.time()) % interval



import random
import string
from typing import TYPE_CHECKING

from rtty_soda.interfaces import Writer

if TYPE_CHECKING:
    from pathlib import Path

import click

__all__ = ["CliWriter"]


class CliWriter(Writer):
    def __init__(self, target: Path | None) -> None:
        self.target = target

    @staticmethod
    def write_bytes_atomic(target: Path, data: bytes) -> None:
        temp_name = "".join(random.choices(string.ascii_lowercase, k=10))  # noqa: S311
        temp_path = target.parent / temp_name
        temp_path.write_bytes(data)
        temp_path.replace(target)

    def write_bytes(self, data: bytes) -> None:
        if self.target is None or self.target.stem == "-":
            add_nl = not data.endswith(b"\n")
            click.echo(data, nl=add_nl)
        else:
            if self.target.exists():
                click.confirm(
                    f"Overwrite the output file? ({self.target})",
                    default=False,
                    abort=True,
                )

            self.write_bytes_atomic(self.target, data)

    def write_diag(self, message: str) -> None:
        click.echo(message, err=True)



from typing import TYPE_CHECKING

from nacl.encoding import RawEncoder
from nacl.public import Box, PrivateKey, PublicKey

if TYPE_CHECKING:
    from nacl.utils import EncryptedMessage


__all__ = ["decrypt", "encrypt"]


def encrypt(private: PrivateKey, public: PublicKey, data: bytes) -> EncryptedMessage:
    box = Box(private_key=private, public_key=public)
    return box.encrypt(plaintext=data, encoder=RawEncoder)


def decrypt(private: PrivateKey, public: PublicKey, data: bytes) -> bytes:
    box = Box(private_key=private, public_key=public)
    return box.decrypt(ciphertext=data, encoder=RawEncoder)



from typing import TYPE_CHECKING, BinaryIO, TextIO, cast

from rtty_soda.interfaces import Reader

if TYPE_CHECKING:
    from pathlib import Path

import click

__all__ = ["CliReader"]


class CliReader(Reader):
    def __init__(self, source: Path) -> None:
        self.source = source

    def read_str(self) -> str:
        with click.open_file(
            self.source, mode="rt", encoding="utf-8", errors="strict"
        ) as fd:
            return cast("TextIO", fd).read()

    def read_bytes(self) -> bytes:
        with click.open_file(self.source, mode="rb") as fd:
            return cast("BinaryIO", fd).read()



from typing import TYPE_CHECKING

from rtty_soda.encoders import ENCODERS

from .service import Service

if TYPE_CHECKING:
    from rtty_soda.formatters import Formatter
    from rtty_soda.interfaces import Reader, Writer

__all__ = ["EncodingService"]


class EncodingService(Service):
    def __init__(
        self,
        in_encoding: str,
        out_encoding: str,
        formatter: Formatter,
        writer: Writer,
        verbose: bool,
    ) -> None:
        super().__init__(formatter, writer, verbose)
        self.in_encoder = ENCODERS.get(in_encoding)
        self.out_encoder = ENCODERS.get(out_encoding)

    def encode(self, data: Reader) -> None:
        buff = self.read_input(data, self.in_encoder)
        self.write_output(buff, self.out_encoder)



from .encoding_service import EncodingService
from .encryption_service import EncryptionService, Keypair, Pipe
from .key_service import KeyService
from .service import FormattedData, Service

__all__ = [
    "EncodingService",
    "EncryptionService",
    "FormattedData",
    "KeyService",
    "Keypair",
    "Pipe",
    "Service",
]



from .encoder import Encoder
from .functions import int_to_bytes

__all__ = ["Base10Encoder"]


class Base10Encoder(Encoder):
    @staticmethod
    def encode(data: bytes) -> str:
        return str(int.from_bytes(data, byteorder="big", signed=False))

    @staticmethod
    def decode(data: str) -> bytes:
        return int_to_bytes(int(data))



from pathlib import Path
from typing import TYPE_CHECKING

import click

from rtty_soda.archivers import ARCHIVERS
from rtty_soda.cryptography.kdf import KDF_PROFILES
from rtty_soda.encoders import ENCODERS

if TYPE_CHECKING:
    from click import ParamType

__all__ = ["CliTypes"]


class CliTypes:
    IN_PATH: ParamType = click.Path(
        exists=True,
        file_okay=True,
        dir_okay=False,
        readable=True,
        allow_dash=True,
        path_type=Path,
    )

    OUT_PATH: ParamType = click.Path(
        file_okay=True, dir_okay=False, writable=True, allow_dash=True, path_type=Path
    )

    ENCODING: ParamType = click.Choice(ENCODERS.keys(), case_sensitive=False)
    COMPRESSION: ParamType = click.Choice(ARCHIVERS.keys(), case_sensitive=False)
    KDF_PROFILE: ParamType = click.Choice(KDF_PROFILES.keys(), case_sensitive=False)
